/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Runtime.Serialization;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines a customized exception which is thrown by this component. Whenever error occurs when rendering the
    /// bread crumb trail control, this exception is thrown. Further enhancements may use this exception as the base
    /// exception of a particular error.
    /// </summary>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [Serializable]
    public class BreadCrumbException : ApplicationException
    {
        /// <summary>
        /// Creates a new instance of <c>BreadCrumbException</c> class. There is no error messages nor original causes.
        /// </summary>
        public BreadCrumbException()
        {
        }

        /// <summary>
        /// Creates a new instance of <c>BreadCrumbException</c> class. The detailed error message is given.
        /// </summary>
        /// <param name="message">The detailed error message.</param>
        public BreadCrumbException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Creates a new instance of <c>BreadCrumbException</c> class. The detailed error message and the original
        /// cause of the error are given.
        /// </summary>
        /// <param name="message">The detailed error message.</param>
        /// <param name="cause">The original cause of the error.</param>
        public BreadCrumbException(string message, Exception cause)
            : base(message, cause)
        {
        }

        /// <summary>
        /// Creates a new instance of <c>BreadCrumbException</c> class. The serialized information used to recreate
        /// the exception is given.
        /// </summary>
        /// <param name="info">The serialization information used to create the exception.</param>
        /// <param name="context">The serialized content used to create the exception.</param>
        protected BreadCrumbException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
