/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines an interface with the functionality to get the current bread crumb node according to the title and
    /// the page context.
    /// </summary>
    /// <remarks>
    /// The concrete implementations should be thread-safe in rendering stage.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public interface INodeDiscovery
    {
        /// <summary>
        /// Gets the bread crumb node representing the given page context. The title of the bread crumb node may be
        /// given.
        /// </summary>
        /// <remarks>
        /// The title of the returned node should be the given title. If there is any query string in the URL of the
        /// bread crumb node, the query strings should be sorted according to its name.
        /// </remarks>
        /// <param name="title">The title of the page.</param>
        /// <param name="page">The page context used to create the bread crumb node.</param>
        /// <returns>The bread crumb node representation of the given page context using the given title.</returns>
        /// <exception cref="ArgumentNullException"><c>page</c> is <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Any error occurs when retrieving the bread crumb node representation.
        /// </exception>
        BreadCrumbNode GetCurrentNode(string title, Page page);
    }
}

