/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines an interface with the functionality to retrieve the bread crumb trail from the given node.
    /// </summary>
    /// <remarks>
    /// The concrete implementations thread-safe in rendering stage.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public interface IPathDiscovery
    {
        /// <summary>
        /// Gets the bread crumb trail of the given current bread crumb node. The page context of the current bread
        /// crumb node is also given.
        /// </summary>
        /// <remarks>
        /// The given bread crumb node may have no title (title as <c>null</c>). When a bread crumb trail can be found
        /// for the given node, the title of the given node should be assigned if it is <c>null</c>. The first node of
        /// the returned trail should be the root node. The last node of the returned trail should be the given node.
        /// If a trail cannot be found, <c>null</c> should be returned. Note that the URL of the given node may contain
        /// query strings. The query strings should be preserved in the returned trail.
        /// </remarks>
        /// <param name="currentNode">The bread crumb node whose trail is retrieved.</param>
        /// <param name="page">The page context represented the bread crumb node.</param>
        /// <returns><c>null</c> if a bread crumb trail cannot be found; otherwise, the bread crumb trail for the node.
        /// </returns>
        /// <exception cref="ArgumentNullException"><c>page</c> is <c>null</c>; or the URL of <c>currentNode</c> is
        /// <c>null</c>.</exception>
        /// <exception cref="BreadCrumbException">Any error occurs when retrieving the trail.</exception>
        BreadCrumbNode[] GetPath(BreadCrumbNode currentNode, Page page);
    }
}

