/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines an interface with the functionality to render the bread crumb trail of the current page to the HTML
    /// web page.
    /// </summary>
    /// <remarks>
    /// The concrete implementations should be thread-safe in rendering stage.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public interface IPathWriter
    {
        /// <summary>
        /// Renders the bread crumb trail to the HTML web page. The given bread crumb node array is from root node to
        /// current node. The node with index 0 is the root node.
        /// </summary>
        /// <remarks>
        /// Implementations should handle the exceptions according to the error disposition flag in configured in the
        /// control. When the error is ignored, the process should continue. When the error is rendered, the exception
        /// should be returned finally while the process should continue. When the error is thrown, the exception
        /// should be returned immediately with no further process.
        /// </remarks>
        /// <param name="writer">The HTML web page writer where the trail is rendered.</param>
        /// <param name="nodes">The bread crumb trail to be rendered.</param>
        /// <param name="parent">The bread crumb trail control containing the configuration.</param>
        /// <returns><c>null</c> if there is no exception; or exceptions which occur during the rendering.</returns>
        /// <exception cref="ArgumentNullException"><c>writer</c>, <c>nodes</c> or <c>parent</c> is <c>null</c>.
        /// </exception>
        /// <exception cref="ArgumentException"><c>nodes</c> is empty array; or <c>nodes</c> contains <c>null</c> URL.
        /// </exception>
        BreadCrumbException[] WritePath(HtmlTextWriter writer, BreadCrumbNode[] nodes, BreadCrumbTrailControl parent);
    }
}

