/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System.Collections;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Defines an interface with the functionality to retrieve the structure of the web site from persistence.
    /// </summary>
    /// <remarks>
    /// The concrete implementations should be thread-safe in rendering stage.
    /// </remarks>
    /// <author>Pops</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public interface ISiteDiscovery
    {
        /// <summary>
        /// Retrieves the structure of the web site from persistence.
        /// </summary>
        /// <remarks>
        /// The structure of the web site is retrieved as a site map. The keys of the site map are <c>INodeMatcher</c>
        /// instances. The values of the site map are <c>IList</c> instances containing <c>INodeMatcher</c> instances.
        /// The list value represents all the logical children of key. When a node matcher has no child, its
        /// corresponding value is an empty list.
        /// </remarks>
        /// <returns>The site map retrieved from persistence.</returns>
        /// <exception cref="BreadCrumbException">Any error occurs when retrieving the structure of the web site.
        /// </exception>
        IDictionary GetSiteMap();
    }
}

