/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Data;
using System.Collections;
using NUnit.Framework;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Tests the functionality and error cases of <c>BreadCrumbTrailControl</c> class. This class only tests the
    /// methods which can be tested directly without using NUnitAsp. When testing the protected method, reflection is
    /// used to invoke such method.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class BreadCrumbTrailControlTestCase
    {
        /// <summary>
        /// Represents the configuration manager used in tests.
        /// </summary>
        private static readonly ConfigManager CONFIG = ConfigManager.GetInstance();

        /// <summary>
        /// Represents the <c>BreadCrumbTrailControl</c> instance used in tests.
        /// </summary>
        private BreadCrumbTrailControl control;

        /// <summary>
        /// Sets up the test environment. The configurations are cleared. A test instance with no configuration is
        /// created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            CONFIG.Clear();
            control = new BreadCrumbTrailControl();
        }

        /// <summary>
        /// Cleans up the test environment. The configurations are cleared. The test instance is disposed.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            control.Dispose();
            control = null;
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparator</c> getter. The default value should be ':'.
        /// </summary>
        [Test]
        public void TestGetPathSeparatorDefaultAccuracy()
        {
            Assert.AreEqual(":", control.PathSeparator, "The default path separator should be ':'.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparator</c> setter when the given value is <c>null</c>. The path separator
        /// should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathSeparatorNullAccuracy()
        {
            control.PathSeparator = null;

            Assert.IsNull(control.PathSeparator, "The path separator should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparator</c> setter when the given value is empty string. The path separator
        /// should be set as given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathSeparatorEmptyAccuracy()
        {
            control.PathSeparator = "  ";

            Assert.AreEqual("  ", control.PathSeparator, "The path separator should be set as empty string.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparator</c> setter for normal case. The path separator should be set as given.
        /// This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathSeparatorNormalAccuracy()
        {
            control.PathSeparator = "valid";

            Assert.AreEqual("valid", control.PathSeparator, "The path separator should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparatorStyle</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetPathSeparatorStyleDefaultAccuracy()
        {
            Assert.IsNull(control.PathSeparatorStyle, "The default path separator style should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparatorStyle</c> setter when the given value is <c>null</c>. The path separator
        /// style should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathSeparatorStyleNullAccuracy()
        {
            control.PathSeparatorStyle = null;

            Assert.IsNull(control.PathSeparatorStyle, "The path separator style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparatorStyle</c> setter when the given value is empty string. The path separator
        /// style should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathSeparatorStyleEmptyAccuracy()
        {
            control.PathSeparatorStyle = "  ";

            Assert.IsNull(control.PathSeparatorStyle, "The path separator style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathSeparatorStyle</c> setter for normal case. The path separator style should be set
        /// as given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathSeparatorStyleNormalAccuracy()
        {
            control.PathSeparatorStyle = "valid";

            Assert.AreEqual("valid", control.PathSeparatorStyle, "The path separator style should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeStyle</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetNodeStyleDefaultAccuracy()
        {
            Assert.IsNull(control.NodeStyle, "The default node style should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeStyle</c> setter when the given value is <c>null</c>. The node style should be set
        /// as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetNodeStyleNullAccuracy()
        {
            control.NodeStyle = null;

            Assert.IsNull(control.NodeStyle, "The node style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeStyle</c> setter when the given value is empty string. The node style should be set
        /// as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetNodeStyleEmptyAccuracy()
        {
            control.NodeStyle = "  ";

            Assert.IsNull(control.NodeStyle, "The node style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeStyle</c> setter for normal case. The node style should be set as given. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetNodeStyleNormalAccuracy()
        {
            control.NodeStyle = "valid";

            Assert.AreEqual("valid", control.NodeStyle, "The node style should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>CurrentNodeStyle</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetCurrentNodeStyleDefaultAccuracy()
        {
            Assert.IsNull(control.CurrentNodeStyle, "The default current node style should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>CurrentNodeStyle</c> setter when the given value is <c>null</c>. The current node style
        /// should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetCurrentNodeStyleNullAccuracy()
        {
            control.CurrentNodeStyle = null;

            Assert.IsNull(control.CurrentNodeStyle, "The current node style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>CurrentNodeStyle</c> setter when the given value is empty string. The current node
        /// style should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetCurrentNodeStyleEmptyAccuracy()
        {
            control.CurrentNodeStyle = "  ";

            Assert.IsNull(control.CurrentNodeStyle, "The current node style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>CurrentNodeStyle</c> setter for normal case. The current node style should be set as
        /// given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetCurrentNodeStyleNormalAccuracy()
        {
            control.CurrentNodeStyle = "valid";

            Assert.AreEqual("valid", control.CurrentNodeStyle, "The current node style should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>RootNodeStyle</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetRootNodeStyleDefaultAccuracy()
        {
            Assert.IsNull(control.RootNodeStyle, "The default root node style should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>RootNodeStyle</c> setter when the given value is <c>null</c>. The root node style
        /// should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetRootNodeStyleNullAccuracy()
        {
            control.RootNodeStyle = null;

            Assert.IsNull(control.RootNodeStyle, "The current node style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>RootNodeStyle</c> setter when the given value is empty string. The root node
        /// style should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetRootNodeStyleEmptyAccuracy()
        {
            control.RootNodeStyle = "  ";

            Assert.IsNull(control.RootNodeStyle, "The root node style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>RootNodeStyle</c> setter for normal case. The root node style should be set as
        /// given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetRootNodeStyleNormalAccuracy()
        {
            control.RootNodeStyle = "valid";

            Assert.AreEqual("valid", control.RootNodeStyle, "The root node style should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>MouseOverStyle</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetMouseOverStyleDefaultAccuracy()
        {
            Assert.IsNull(control.MouseOverStyle, "The default root node style should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>MouseOverStyle</c> setter when the given value is <c>null</c>. The mouse over style
        /// should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetMouseOverStyleNullAccuracy()
        {
            control.MouseOverStyle = null;

            Assert.IsNull(control.MouseOverStyle, "The mouse over style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>MouseOverStyle</c> setter when the given value is empty string. The mouse over
        /// style should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetMouseOverStyleEmptyAccuracy()
        {
            control.MouseOverStyle = "  ";

            Assert.IsNull(control.MouseOverStyle, "The mouse over style should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>MouseOverStyle</c> setter for normal case. The mouse over style should be set as
        /// given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetMouseOverStyleNormalAccuracy()
        {
            control.MouseOverStyle = "valid";

            Assert.AreEqual("valid", control.MouseOverStyle, "The mouse over style should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>ErrorDisposition</c> getter for the default value. The default value should be
        /// <c>SilentlyIgnore</c>.
        /// </summary>
        [Test]
        public void TestGetErrorDispositionDefaultAccuracy()
        {
            Assert.AreEqual(ErrorDisposition.SilentlyIgnore, control.ErrorDisposition,
                "The default error disposition should be silently ignore.");
        }

        /// <summary>
        /// Tests <c>ErrorDisposition</c> setter when the given value is not a valid error disposition flag. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestSetErrorDispositionInvalid()
        {
            control.ErrorDisposition = (ErrorDisposition)3;
        }

        /// <summary>
        /// Tests accuracy of <c>ErrorDisposition</c> setter. The error disposition flag should be set as given. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetErrorDispositionAccuracy()
        {
            control.ErrorDisposition = ErrorDisposition.ThrowException;

            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathDirection</c> getter for the default value. The default value should be
        /// <c>RootToCurrent</c>.
        /// </summary>
        [Test]
        public void TestGetPathDirectionDefaultAccuracy()
        {
            Assert.AreEqual(PathDirection.RootToCurrent, control.PathDirection,
                "The default path direction should be from root to current.");
        }

        /// <summary>
        /// Tests <c>PathDirection</c> setter when the given value is not a valid path direction flag. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestSetPathDirectionInvalid()
        {
            control.PathDirection = (PathDirection)2;
        }

        /// <summary>
        /// Tests accuracy of <c>PathDirection</c> setter. The path direction flag should be set as given. This is
        /// verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathDirectionAccuracy()
        {
            control.PathDirection = PathDirection.CurrentToRoot;

            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>Title</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetTitleDefaultAccuracy()
        {
            Assert.IsNull(control.Title, "The default title should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Title</c> setter when the given value is <c>null</c>. The title should be set as
        /// <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetTitleNullAccuracy()
        {
            control.Title = null;

            Assert.IsNull(control.Title, "The title should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Title</c> setter when the given value is empty string. The title should be set as
        /// <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetTitleEmptyAccuracy()
        {
            control.Title = "  ";

            Assert.IsNull(control.Title, "The title should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>Title</c> setter for normal case. The title should be set as given. This is verified by
        /// the property getter.
        /// </summary>
        [Test]
        public void TestSetTitleNormalAccuracy()
        {
            control.Title = "valid";

            Assert.AreEqual("valid", control.Title, "The title should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeDiscoveryKey</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetNodeDiscoveryKeyDefaultAccuracy()
        {
            Assert.IsNull(control.NodeDiscoveryKey, "The default node discovery key should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeDiscoveryKey</c> setter when the given value is <c>null</c>. The node discovery key
        /// should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetNodeDiscoveryKeyNullAccuracy()
        {
            control.NodeDiscoveryKey = null;

            Assert.IsNull(control.NodeDiscoveryKey, "The node discovery key should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeDiscoveryKey</c> setter when the given value is empty string. The node discovery
        /// key should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetNodeDiscoveryKeyEmptyAccuracy()
        {
            control.NodeDiscoveryKey = "  ";

            Assert.IsNull(control.NodeDiscoveryKey, "The node discovery key should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>NodeDiscoveryKey</c> setter for normal case. The node discovery key should be set as
        /// given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetNodeDiscoveryKeyNormalAccuracy()
        {
            control.NodeDiscoveryKey = "valid";

            Assert.AreEqual("valid", control.NodeDiscoveryKey, "The node discovery key should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathDiscoveryKey</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetPathDiscoveryKeyDefaultAccuracy()
        {
            Assert.IsNull(control.PathDiscoveryKey, "The default path discovery key should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathDiscoveryKey</c> setter when the given value is <c>null</c>. The path discovery key
        /// should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathDiscoveryKeyNullAccuracy()
        {
            control.PathDiscoveryKey = null;

            Assert.IsNull(control.PathDiscoveryKey, "The node discovery key should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathDiscoveryKey</c> setter when the given value is empty string. The path discovery
        /// key should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathDiscoveryKeyEmptyAccuracy()
        {
            control.PathDiscoveryKey = "  ";

            Assert.IsNull(control.PathDiscoveryKey, "The path discovery key should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathDiscoveryKey</c> setter for normal case. The path discovery key should be set as
        /// given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathDiscoveryKeyNormalAccuracy()
        {
            control.PathDiscoveryKey = "valid";

            Assert.AreEqual("valid", control.PathDiscoveryKey, "The path discovery key should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathWriterKey</c> getter. The default value should be <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetPathWriterKeyDefaultAccuracy()
        {
            Assert.IsNull(control.PathWriterKey, "The default path writer key should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathWriterKey</c> setter when the given value is <c>null</c>. The path writer key
        /// should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathWriterKeyNullAccuracy()
        {
            control.PathWriterKey = null;

            Assert.IsNull(control.PathWriterKey, "The path writer key should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathWriterKey</c> setter when the given value is empty string. The path writer
        /// key should be set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathWriterKeyEmptyAccuracy()
        {
            control.PathWriterKey = "  ";

            Assert.IsNull(control.PathWriterKey, "The path writer key should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>PathWriterKey</c> setter for normal case. The path writer key should be set as
        /// given. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetPathWriterKeyNormalAccuracy()
        {
            control.PathWriterKey = "valid";

            Assert.AreEqual("valid", control.PathWriterKey, "The path writer key should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>DataSource</c> getter for the default value. The default value is <c>null</c>.
        /// </summary>
        [Test]
        public void TestGetDataSourceDefaultAccuracy()
        {
            Assert.IsNull(control.DataSource, "The default data source is null.");
        }

        /// <summary>
        /// Tests <c>DataSource</c> setter when the data source is not a data set, a data table nor a list. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestSetDataSourceInvalidType()
        {
            control.DataSource = new object();
        }

        /// <summary>
        /// Tests accuracy of <c>DataSource</c> setter when the data source is <c>null</c>. The data source should be
        /// set as <c>null</c>. This is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetDataSourceNullAccuracy()
        {
            control.DataSource = null;

            Assert.IsNull(control.DataSource, "The data source should be set as null.");
        }

        /// <summary>
        /// Tests accuracy of <c>DataSource</c> setter for normal cases. The data source should be set as given. This
        /// is verified by the property getter.
        /// </summary>
        [Test]
        public void TestSetDataSourceNormalAccuracy()
        {
            DataTable expected = new DataTable();

            control.DataSource = expected;

            Assert.AreSame(expected, control.DataSource, "The data source should be set as given.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when all property configurations are valid. The properties
        /// should be able to be retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlValidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/Valid.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'pathSeparator' is invalid. Other properties
        /// should still be loaded while this property uses the default value. The properties should be able to be
        /// retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlPathSeparatorInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoPathSeparator.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual(":", control.PathSeparator, "The path separator should be the default value.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'pathSeparatorStyle' is invalid. Other
        /// properties should still be loaded while this property uses the default value. The properties should be able
        /// to be retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlPathSeparatorStyleInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoPathSeparatorStyle.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.IsNull(control.PathSeparatorStyle, "The path separator style should be the default value.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'nodeStyle' is invalid. Other properties
        /// should still be loaded while this property uses the default value. The properties should be able to be
        /// retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlNodeStyleInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoNodeStyle.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.IsNull(control.NodeStyle, "The node style should be the default value.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'rootNodeStyle' is invalid. Other properties
        /// should still be loaded while this property uses the default value. The properties should be able to be
        /// retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlRootNodeStyleInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoRootNodeStyle.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.IsNull(control.RootNodeStyle, "The root node style should be the default value.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'currentNodeStyle' is invalid. Other
        /// properties should still be loaded while this property uses the default value. The properties should be
        /// able to be retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlCurrentNodeStyleInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoCurrentNodeStyle.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.IsNull(control.CurrentNodeStyle, "The current node style should be the default value.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'mouseOverStyle' is invalid. Other properties
        /// should still be loaded while this property uses the default value. The properties should be able to be
        /// retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlMouseOverStyleInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoMouseOverStyle.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.IsNull(control.MouseOverStyle, "The mouse over style should be the default value.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'pathDirection' is invalid. Other properties
        /// should still be loaded while this property uses the default value. The properties should be able to be
        /// retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlPathDirectionInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/InvalidPathDirection.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.RootToCurrent, control.PathDirection,
                "The path direction flag should be the default value.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'errorDisposition' is invalid. Other
        /// properties should still be loaded while this property uses the default value. The properties should be able
        /// to be retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlErrorDispositionInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/InvalidErrorDisposition.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.SilentlyIgnore, control.ErrorDisposition,
                "The error disposition flag should be the default value.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'nodeDiscoveryKey' is invalid. Other
        /// properties should still be loaded while this property uses the default value. The properties should be able
        /// to be retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlNodeDiscoveryKeyInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoNodeDiscoveryKey.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.IsNull(control.NodeDiscoveryKey, "The node discovery key should be the default value.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'pathDiscoveryKey' is invalid. Other
        /// properties should still be loaded while this property uses the default value. The properties should be able
        /// to be retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlPathDiscoveryKeyInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoPathDiscoveryKey.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.IsNull(control.PathDiscoveryKey, "The path discovery key should be the default value.");
            Assert.AreEqual("path writer key", control.PathWriterKey, "The path writer key should be loaded.");
        }

        /// <summary>
        /// Tests accuracy of <c>BreadCrumbTrailControl</c> when property 'pathWriterKey' is invalid. Other properties
        /// should still be loaded while this property uses the default value. The properties should be able to be
        /// retrieved via the property getters. This is the way to verify the accuracy.
        /// </summary>
        [Test]
        public void TestBreadCrumbTrailControlPathWriterKeyInvalidAccuracy()
        {
            control.Dispose();

            // Load the configuration
            CONFIG.LoadFile("../../test_files/Control/NoPathWriterKey.xml");

            // Create a new instance
            control = new BreadCrumbTrailControl();

            // Verify
            Assert.AreEqual("path separator", control.PathSeparator, "The path separator should be loaded.");
            Assert.AreEqual("path separator style", control.PathSeparatorStyle,
                "The path separator style should be loaded.");
            Assert.AreEqual("node style", control.NodeStyle, "The node style should be loaded.");
            Assert.AreEqual("root node style", control.RootNodeStyle, "The root node style should be loaded.");
            Assert.AreEqual("current node style", control.CurrentNodeStyle, "The current node style should be loaded.");
            Assert.AreEqual("mouse over style", control.MouseOverStyle, "The mouse over style should be loaded.");
            Assert.AreEqual(PathDirection.CurrentToRoot, control.PathDirection,
                "The path direction flag should be from current to root.");
            Assert.AreEqual(ErrorDisposition.ThrowException, control.ErrorDisposition,
                "The error disposition flag should be throwing exception.");
            Assert.AreEqual("node discovery key", control.NodeDiscoveryKey, "The node discovery key should be loaded.");
            Assert.AreEqual("path discovery key", control.PathDiscoveryKey, "The path discovery key should be loaded.");
            Assert.IsNull(control.PathWriterKey, "The path writer key should be the default value.");
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a data set, but it contains no data table. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindDataSetNoTable()
        {
            control.DataSource = new DataSet();

            control.DataBind();
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a data table, but does not contain column 'Title' or 'Url'.
        /// Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindDataTableInvalidColumn()
        {
            // Build the data table
            DataTable table = new DataTable();

            table.Columns.Add(new DataColumn("Title", typeof(string)));
            table.Rows.Add(new object[] { "title" });

            // Set the data table as the data source
            control.DataSource = table;

            // Bind the data
            control.DataBind();
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a data table, and it contains column 'Title' and 'Url'.
        /// However, the type of the two columns are not strings. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindDataTableInvalidType()
        {
            // Build the data table
            DataTable table = new DataTable();

            table.Columns.Add(new DataColumn("Title", typeof(string)));
            table.Columns.Add(new DataColumn("Url", typeof(bool)));
            table.Rows.Add(new object[] { "title", false });

            // Set the data table as the data source
            control.DataSource = table;

            // Bind the data
            control.DataBind();
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a data table, and it contains column 'Title' and 'Url'.
        /// However, one row contains empty string. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindDataTableInvalidRow()
        {
            // Build the data table
            DataTable table = new DataTable();

            table.Columns.Add(new DataColumn("Title", typeof(string)));
            table.Columns.Add(new DataColumn("Url", typeof(string)));
            table.Rows.Add(new object[] { "title", "    " });

            // Set the data table as the data source
            control.DataSource = table;

            // Bind the data
            control.DataBind();
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a list, but it is empty. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindIListEmpty()
        {
            control.DataSource = new object[0];

            control.DataBind();
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a list, but it contains <c>null</c>. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindIListContainNull()
        {
            control.DataSource = new object[] { null };

            control.DataBind();
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a list, but it contains a node with <c>null</c> URL. Expect
        /// ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindIListContainNullURL()
        {
            control.DataSource = new object[] { new BreadCrumbNode() };

            control.DataBind();
        }

        /// <summary>
        /// Tests <c>DataBind()</c> when the data source is a list, but it contains instances other than
        /// <c>BreadCrumbNode</c>. Expect ArgumentException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestDataBindIListContainOtherType()
        {
            control.DataSource = new object[] { new object() };

            control.DataBind();
        }

        /// <summary>
        /// Tests accuracy of <c>DataBind()</c> when the first data table in the data set is empty. The overridden path
        /// should be cleared. This is verified via reflection.
        /// </summary>
        [Test]
        public void TestDataBindDataSetEmptyTableAccuracy()
        {
            // Create the data set
            DataSet dataSet = new DataSet();
            dataSet.Tables.Add(new DataTable());

            // Set the data source
            control.DataSource = dataSet;

            // Bind the data
            control.DataBind();

            // Verify
            Assert.IsNull(TestHelper.GetField(control, "pathOverride"), "The overridden path should be cleared.");
        }

        /// <summary>
        /// Tests accuracy of <c>DataBind()</c> when the data source is a data set. The first table in the data set
        /// should be used. The overridden path should be the same as a reference one. This is verified via reflection.
        /// </summary>
        [Test]
        public void TestDataBindDataSetAccuracy()
        {
            // Create the data set
            DataSet dataSet = new DataSet();
            DataTable table = new DataTable();
            dataSet.Tables.Add(table);
            dataSet.Tables.Add(new DataTable());

            // Create the data table
            table.Columns.Add(new DataColumn("Title", typeof(string)));
            table.Columns.Add(new DataColumn("Url", typeof(string)));
            table.Rows.Add(new object[] { "root", "root.aspx" });
            table.Rows.Add(new object[] { "node1", "node1.aspx" });
            table.Rows.Add(new object[] { "node2", "node2.aspx" });
            table.Rows.Add(new object[] { "current", "current.aspx" });

            // Set the data source
            control.DataSource = dataSet;

            // Bind the data
            control.DataBind();

            // Build the expected result
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx"),
                new BreadCrumbNode("current", "current.aspx")
            };

            // Verify
            TestHelper.AreEqual(nodes, TestHelper.GetField(control, "pathOverride"),
                "The overridden path should be correct.");
        }

        /// <summary>
        /// Tests accuracy of <c>DataBind()</c> when the data table is empty. The overridden path should be cleared.
        /// This is verified via reflection.
        /// </summary>
        [Test]
        public void TestDataBindDataTableEmptyAccuracy()
        {
            // Set the data source
            control.DataSource = new DataTable();

            // Bind the data
            control.DataBind();

            // Verify
            Assert.IsNull(TestHelper.GetField(control, "pathOverride"), "The overridden path should be cleared.");
        }

        /// <summary>
        /// Tests accuracy of <c>DataBind()</c> when the data source is a data table. The overridden path should be the
        /// same as a reference one. This is verified via reflection.
        /// </summary>
        [Test]
        public void TestDataBindDataTableAccuracy()
        {
            // Create the data table
            DataTable table = new DataTable();

            table.Columns.Add(new DataColumn("Title", typeof(string)));
            table.Columns.Add(new DataColumn("Url", typeof(string)));
            table.Rows.Add(new object[] { "root", "root.aspx" });
            table.Rows.Add(new object[] { "node1", "node1.aspx" });
            table.Rows.Add(new object[] { "node2", "node2.aspx" });
            table.Rows.Add(new object[] { "current", "current.aspx" });

            // Set the data source
            control.DataSource = table;

            // Bind the data
            control.DataBind();

            // Build the expected result
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx"),
                new BreadCrumbNode("current", "current.aspx")
            };

            // Verify
            TestHelper.AreEqual(nodes, TestHelper.GetField(control, "pathOverride"),
                "The overridden path should be correct.");
        }

        /// <summary>
        /// Tests accuracy of <c>DataBind()</c> when the data source is a list. A copy of the list should be used as
        /// the overridden path. This is verified via reflection.
        /// </summary>
        [Test]
        public void TestDataBindIListAccuracy()
        {
            // Build the expected result
            BreadCrumbNode[] nodes = new BreadCrumbNode[] {
                new BreadCrumbNode("root", "root.aspx"),
                new BreadCrumbNode("node1", "node1.aspx"),
                new BreadCrumbNode("node2", "node2.aspx"),
                new BreadCrumbNode("current", "current.aspx")
            };

            // Set the data source
            control.DataSource = nodes;

            // Bind the data
            control.DataBind();

            // Verify
            TestHelper.AreEqual(nodes, TestHelper.GetField(control, "pathOverride"),
                "The overridden path should be correct.");
            Assert.IsTrue(nodes != TestHelper.GetField(control, "pathOverride"), "A copy should be made.");
        }

        /// <summary>
        /// Tests <c>RenderContents(HtmlTextWriter)</c> when the given HTML writer is <c>null</c>. Expect
        /// ArgumentNullException.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestRenderContentsHtmlTextWriterNull()
        {
            TestHelper.Invoke(control, "RenderContents", new object[] { null });
        }
    }
}

