/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System.Web.UI;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Defines a mock concrete implementation of <c>INodeDiscovery</c>. It simply uses the title as the URL.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0</version>
    /// <copyright></copyright>
    public class MockNodeDiscovery : INodeDiscovery
    {
        /// <summary>
        /// Creates a new instance of <c>MockNodeDiscovery</c> class.
        /// </summary>
        public MockNodeDiscovery()
        {
        }

        /// <summary>
        /// Gets the bread crumb node representing the given page context. The title of the bread crumb node may be
        /// given.
        /// </summary>
        /// <remarks>
        /// The title is used as the URL. When the title is 'error', it throws an exception.
        /// </remarks>
        /// <param name="title">The title of the page.</param>
        /// <param name="page">The page context used to create the bread crumb node.</param>
        /// <returns>The bread crumb node representation of the given page context using the given title.</returns>
        /// <exception cref="BreadCrumbException">The title is 'error'.</exception>
        public BreadCrumbNode GetCurrentNode(string title, Page page)
        {
            if (title == "error")
            {
                throw new BreadCrumbException("Get Node Error");
            }

            return new BreadCrumbNode(title, title);
        }
    }
}

