<%@ Page Language="C#" Debug="true" %>

<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="TopCoder.Web.UI.WebControl.BreadCrumb" %>
<%@ Import Namespace="TopCoder.Util.ConfigurationManager" %>
<%@ Import Namespace="NUnit.Framework" %>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN">

<script runat="server">
    /// <summary>
    /// Represents the <c>BreadCrumbTrailControl</c> instance used in tests.
    /// </summary>
    private BreadCrumbTrailControl control;

    /// <summary>
    /// Represents the string writer used in tests.
    /// </summary>
    private TextWriter writer;

    /// <summary>
    /// Represents the HTML text writer used in tests.
    /// </summary>
    private HtmlTextWriter htmlWriter;

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when an exception is thrown before writing the path.
    /// The error disposition is SilentlyIgnore, so the error is ignored. There should be nothing rendered.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterErrorBeforeWriteIgnoreAccuracy(object sender, EventArgs e)
    {
        // Set the error disposition flag
        control.ErrorDisposition = ErrorDisposition.SilentlyIgnore;

        // Set the title as error, so that an exception is generated.
        control.Title = "error";

        control.RenderControl(htmlWriter);

        htmlWriter.Flush();

        // Verify
        Assert.AreEqual("<span></span>", writer.ToString(), "There should be no rendering done.");
    }

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when an exception is thrown before writing the path.
    /// The error disposition is RenderException, so the error is rendered.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterErrorBeforeWriteRenderAccuracy(object sender, EventArgs e)
    {
        // Set the error disposition flag
        control.ErrorDisposition = ErrorDisposition.RenderException;

        // Set the title as error, so that an exception is generated.
        control.Title = "error";

        control.RenderControl(htmlWriter);

        htmlWriter.Flush();

        // Verify
        Assert.IsTrue(writer.ToString().IndexOf("TopCoder.Web.UI.WebControl.BreadCrumb.BreadCrumbException") >= 0,
            "The error should be rendered.");
    }

    /// <summary>
    /// Tests <c>RenderContents(HtmlTextWriter)</c> when an exception is thrown before writing the path. The error
    /// disposition is ThrowException, so the error is thrown. Expect BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterErrorBeforeWriteThrow(object sender, EventArgs e)
    {
        // Set the error disposition flag
        control.ErrorDisposition = ErrorDisposition.ThrowException;

        // Set the title as error, so that an exception is generated.
        control.Title = "error";

        try
        {
            control.RenderControl(htmlWriter);
            Assert.Fail("An error occurs, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when exceptions occur during writing the path. The
    /// error disposition is SilentlyIgnore, so the errors are ignored. It should render as much as possible.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterWriteErrorIgnoreAccuracy(object sender, EventArgs e)
    {
        // Set the error disposition flag
        control.ErrorDisposition = ErrorDisposition.SilentlyIgnore;

        // Set the title as error, so that an exception is generated.
        control.Title = "current";
        control.PathWriterKey = "ErrorWriter";

        control.RenderControl(htmlWriter);

        htmlWriter.Flush();

        // Verify
        Assert.IsTrue(writer.ToString().
            IndexOf("$root|root||%<span>:</span>$node1|node1||%<span>:</span>$current|current||%") >= 0,
            "The path is rendered as much as possible.");
    }

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when exceptions occur during writing the path. The
    /// error disposition is RenderException, so the errors are rendered. It should render as much as possible.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterWriteErrorRenderAccuracy(object sender, EventArgs e)
    {
        // Set the error disposition flag
        control.ErrorDisposition = ErrorDisposition.RenderException;

        // Set the title as error, so that an exception is generated.
        control.Title = "current";
        control.PathWriterKey = "ErrorWriter";

        control.RenderControl(htmlWriter);

        htmlWriter.Flush();

        // Verify
        Assert.IsTrue(writer.ToString().IndexOf("Begin Error") >= 0, "All errors should be rendered.");
        Assert.IsTrue(writer.ToString().IndexOf("Node Error") >= 0, "All errors should be rendered.");
        Assert.IsTrue(writer.ToString().IndexOf("Separator Error") >= 0, "All errors should be rendered.");
        Assert.IsTrue(writer.ToString().IndexOf("End Error") >= 0, "All errors should be rendered.");
        Assert.IsTrue(writer.ToString().
            IndexOf("$root|root||%<span>:</span>$node1|node1||%<span>:</span>$current|current||%") >= 0,
            "The path is rendered as much as possible.");
    }

    /// <summary>
    /// Tests <c>RenderContents(HtmlTextWriter)</c> when exceptions occur during writing the path. The error
    /// disposition is ThrowException, so the first error is thrown. Expect BreadCrumbException.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterWriteErrorThrow(object sender, EventArgs e)
    {
        // Set the error disposition flag
        control.ErrorDisposition = ErrorDisposition.ThrowException;

        // Set the title as error, so that an exception is generated.
        control.Title = "current";
        control.PathWriterKey = "ErrorWriter";

        try
        {
            control.RenderControl(htmlWriter);
            Assert.Fail("An error occurs, should throw BreadCrumbException.");
        }
        catch (BreadCrumbException)
        {
            // good
        }
    }

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when there is no overridden path, and all keys are
    /// not specified. It should use default keys to create instances. The path is discovered automatically. This is
    /// verified by comparing the output to a reference output.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterNoOverrideNoKeyAccuracy(object sender, EventArgs e)
    {
        // Set the title
        control.Title = "current";
        
        // Render the control
        control.RenderControl(htmlWriter);
        
        htmlWriter.Flush();

        // Verify
        Assert.AreEqual("<span><a href=\"root\">root</a><span>:</span><a href=\"node1\">node1</a>" +
            "<span>:</span><a href=\"current\">current</a></span>", writer.ToString(),
            "The rendered result should be the same as expected.");
    }

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when there is no overridden path, and all keys are
    /// specified. It should use the specified keys to create instances. The path is discovered automatically. This
    /// is verified by comparing the output to a reference output.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterNoOverrideAllKeysAccuracy(object sender, EventArgs e)
    {
        // Set the title
        control.Title = "current";
        
        // Set the keys
        control.NodeDiscoveryKey = "TopCoder.Web.UI.WebControl.BreadCrumb.INodeDiscovery";
        control.PathDiscoveryKey = "TopCoder.Web.UI.WebControl.BreadCrumb.IPathDiscovery";
        control.PathWriterKey = "TopCoder.Web.UI.WebControl.BreadCrumb.IPathWriter";

        // Render the control
        control.RenderControl(htmlWriter);

        htmlWriter.Flush();

        // Verify
        Assert.AreEqual("<span><a href=\"root\">root</a><span>:</span><a href=\"node1\">node1</a>" +
            "<span>:</span><a href=\"current\">current</a></span>", writer.ToString(),
            "The rendered result should be the same as expected.");
    }

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when there is an overridden path. The path should be
    /// rendered directly. This is verified by comparing the output to a reference output.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterOverrideAccuracy(object sender, EventArgs e)
    {
        // Set the title
        control.Title = "current";

        // Set the data source
        control.DataSource = new BreadCrumbNode[] {
            new BreadCrumbNode("11", "1"),
            new BreadCrumbNode("22", "2")
        };
        
        // Bind the data
        control.DataBind();

        // Render the control
        control.RenderControl(htmlWriter);

        htmlWriter.Flush();

        // Verify
        Assert.AreEqual("<span><a href=\"1\">11</a><span>:</span><a href=\"2\">22</a></span>", writer.ToString(),
            "The rendered result should be the same as expected.");
    }

    /// <summary>
    /// Tests accuracy of <c>RenderContents(HtmlTextWriter)</c> when there is no path for the current node. It
    /// should render nothing.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestRenderContentsHtmlTextWriterNoPathAccuracy(object sender, EventArgs e)
    {
        // Set the title
        control.Title = "unknown";

        // Render the control
        control.RenderControl(htmlWriter);

        htmlWriter.Flush();

        // Verify
        Assert.AreEqual("<span></span>", writer.ToString(), "No path should result nothing.");
    }

    /// <summary>
    /// Sets up the test environment when the page is loaded. The test instances are created. The session is cleared.
    /// The configurations are cleared. A valid configuration is then loaded.
    /// </summary>
    /// <param name="e">The event parameter.</param>
    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        Page.Session.Clear();
        ConfigManager.GetInstance().Clear();
        control = new BreadCrumbTrailControl();
        control.Page = Page;
        ConfigManager.GetInstance().LoadFile(Page.MapPath("ControlConfig/Test.xml"));
        writer = new StringWriter();
        htmlWriter = new HtmlTextWriter(writer);
    }

    /// <summary>
    /// Cleans up the test environment when the page is unloaded. The test instances are disposed. The session is
    /// cleared. The configurations are cleared.
    /// </summary>
    /// <param name="e">The event parameter.</param>
    protected override void OnUnload(EventArgs e)
    {
        htmlWriter.Close();
        htmlWriter = null;
        writer = null;
        control.Dispose();
        control = null;
        ConfigManager.GetInstance().Clear();
        Page.Session.Clear();

        base.OnUnload(e);
    }
</script>

<html xmlns="http://www.w3.org/1999/xhtml">
<head runat="server">
    <title>DynamicPathDiscoveryTest</title>
</head>
<body runat="server">
    <form id="form1" runat="server">
        <div>
            <asp:Button ID="RenderContentsHtmlTextWriterErrorBeforeWriteIgnoreAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterErrorBeforeWriteIgnoreAccuracy" Text="RenderContentsHtmlTextWriterErrorBeforeWriteIgnoreAccuracy" />
            <asp:Button ID="RenderContentsHtmlTextWriterErrorBeforeWriteRenderAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterErrorBeforeWriteRenderAccuracy" Text="RenderContentsHtmlTextWriterErrorBeforeWriteRenderAccuracy" />
            <asp:Button ID="RenderContentsHtmlTextWriterErrorBeforeWriteThrow" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterErrorBeforeWriteThrow" Text="RenderContentsHtmlTextWriterErrorBeforeWriteThrow" />
            <asp:Button ID="RenderContentsHtmlTextWriterWriteErrorIgnoreAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterWriteErrorIgnoreAccuracy" Text="RenderContentsHtmlTextWriterWriteErrorIgnoreAccuracy" />
            <asp:Button ID="RenderContentsHtmlTextWriterWriteErrorRenderAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterWriteErrorRenderAccuracy" Text="RenderContentsHtmlTextWriterWriteErrorRenderAccuracy" />
            <asp:Button ID="RenderContentsHtmlTextWriterWriteErrorThrow" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterWriteErrorThrow" Text="RenderContentsHtmlTextWriterWriteErrorThrow" />
            <asp:Button ID="RenderContentsHtmlTextWriterNoOverrideNoKeyAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterNoOverrideNoKeyAccuracy" Text="RenderContentsHtmlTextWriterNoOverrideNoKeyAccuracy" />
            <asp:Button ID="RenderContentsHtmlTextWriterNoOverrideAllKeysAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterNoOverrideAllKeysAccuracy" Text="RenderContentsHtmlTextWriterNoOverrideAllKeysAccuracy" />
            <asp:Button ID="RenderContentsHtmlTextWriterOverrideAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterOverrideAccuracy" Text="RenderContentsHtmlTextWriterOverrideAccuracy" />
            <asp:Button ID="RenderContentsHtmlTextWriterNoPathAccuracy" runat="server"
                OnClick="TestRenderContentsHtmlTextWriterNoPathAccuracy" Text="RenderContentsHtmlTextWriterNoPathAccuracy" />
        </div>
    </form>
</body>
</html>
