<%@ Page Language="C#" %>
<%@ Import Namespace="TopCoder.Web.UI.WebControl.BreadCrumb" %>
<%@ Import Namespace="TopCoder.Web.UI.WebControl.BreadCrumb.NodeDiscovery" %>
<%@ Import Namespace="NUnit.Framework" %>

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN">

<script runat="server">
    /// <summary>
    /// Represents the <c>HttpRequestNodeDiscovery</c> instance used in tests.
    /// </summary>
    private INodeDiscovery nodeDiscovery;
    
    /// <summary>
    /// Tests accuracy of <c>GetCurrentNode(string, Page)</c> when the HTTP request of the given page does not have
    /// a query string. The returned bread crumb node should have the given title and the URL of the HTTP request.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetCurrentNodeStringPageNoQueryStringAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = nodeDiscovery.GetCurrentNode("title", Page);
        
        // Verify
        Assert.AreEqual("title", node.Title, "The title should be 'title'.");
        Assert.AreEqual("/test/HttpRequestNodeDiscoveryTest.aspx", node.Url,
            "The URL should be the URL of the request with no query string.");
    }

    /// <summary>
    /// Tests accuracy of <c>GetCurrentNode(string, Page)</c> when the HTTP request of the given page has a query
    /// string. The returned bread crumb node should have the given title and the URL of the HTTP request. The query
    /// string should be sorted according to the property names.
    /// </summary>
    /// <param name="sender">The instance which causes this event.</param>
    /// <param name="e">The event parameter.</param>
    protected void TestGetCurrentNodeStringPageQueryStringAccuracy(object sender, EventArgs e)
    {
        BreadCrumbNode node = nodeDiscovery.GetCurrentNode("title", Page);

        // Verify
        Assert.AreEqual("title", node.Title, "The title should be 'title'.");
        Assert.AreEqual("/test/HttpRequestNodeDiscoveryTest.aspx?execute=cmd&print=exe&query=abc",
            node.Url, "The URL should be the URL of the request with query string sorted.");
    }

    /// <summary>
    /// Sets up the test environment when the page is loaded. The test instance is created.
    /// </summary>
    /// <param name="e">The event parameter.</param>
    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        nodeDiscovery = new HttpRequestNodeDiscovery();
    }

    /// <summary>
    /// Cleans up the test environment when the page is unloaded. The test instance is disposed.
    /// </summary>
    /// <param name="e">The event parameter.</param>
    protected override void OnUnload(EventArgs e)
    {
        nodeDiscovery = null;
        
        base.OnUnload(e);
    }
</script>

<html xmlns="http://www.w3.org/1999/xhtml" >
<head runat="server">
    <title>HttpRequestNodeDiscoveryTest</title>
</head>
<body runat="server">
    <form id="form1" runat="server">
    <div>
        <asp:Button ID="GetCurrentNodeStringPageNoQueryStringAccuracy" runat="server" OnClick="TestGetCurrentNodeStringPageNoQueryStringAccuracy" Text="GetCurrentNodeStringPageNoQueryStringAccuracy" />
        <asp:Button ID="GetCurrentNodeStringPageQueryStringAccuracy" runat="server" OnClick="TestGetCurrentNodeStringPageQueryStringAccuracy" Text="GetCurrentNodeStringPageNoQueryStringAccuracy" />
    </div>
    </form>
</body>
</html>
